// Sington class. Recommended usage: Put sth. like
// var p = new Ext.ux.state.BMSProvider({appName:'myAppName',filter:'key filter'}); // the optional filter could be sth. like 'abc%' to retrieve only the key:value pairs with keys starting with abc
// Ext.state.Manager.setProvider(p);
// before calling Ext.onReady, and replace Ext.onReady with p.onReady
// that'll make sure the state values are initialized before any UI components are rendered

Ext.ns('Ext.ux.state');
Ext.ux.state.BMSProvider = function(config) {
  this.parent = Ext.ux.state.BMSProvider.superclass;
  this.parent.constructor.call(this);
  Ext.apply(this, config, {
    appName:document.location.href, // override highly recommended
    getUser:true,                   // true will return BMS user info in this.extraVals.BMSUser (NOT stored in this.state.BMSUser so it's not stored in DB)
    autoInit:true,                  // initiate loading at obj (this) creation? Better not override - if set to false you'd have to call initState yourself (most likely at beginning of your app)!
    url:'/biocgi/pl-shared/state2DB.pl',

    // don't override below!!
    initReady:false,
    docReady:false
  });
  this.decodeValue = Ext.util.JSON.decode;
  this.encodeValue = Ext.util.JSON.encode;
  this.dSet = {};
  if(this.autoInit) this.initState();
};
Ext.extend(Ext.ux.state.BMSProvider, Ext.state.Provider, {
  onReady: function(fn, scope, options)
  {
    this.callback = fn.createDelegate(scope, options);
    Ext.onReady(function(){
      this.docReady = true;
      if(this.initReady) this.callback();
    }, this);
  },
  onInitReady: function(resp, opt)
  {
    var s = Ext.decode(resp.responseText);
    for(key in s)
    {
      var v = {};
      try { v = this.decodeValue(s[key]); }
      catch(err) {
        if(this.errNotifyURL)
        {
          Ext.Ajax.request({
            url: this.errNotifyURL,
            params: {admin:this.errNotifyAdmin || '', msg:err.toString(), url:opt.url, action:opt.params.action, appName:opt.params.appName, filter:opt.params.filter, key:key, badData: s[key]}
          });
        }
      }
      if(key == 'extraVals') this[key] = v;
      else this.state[key] = v;
    }
    this.initReady = true;
    if(this.docReady) this.callback();
  },
  initState: function()
  {
    Ext.Ajax.request({
      url: this.url,
      params: {appName: this.appName, action: 'init', filter: this.filter, getUser: this.getUser},
      success: this.onInitReady,
      failure: function(){this.failed = true; Ext.Msg.alert('Preference Saving Error', 'State service failed - your custom views (column size in grid, window size etc.) will not be loaded or saved')},
      scope: this
    });
  },
  // if one wants to retrieve info little by little using different filters (not right at start), it could be
  // done by using gsFilter to set new filter, then resetState() (pass in callback!), then call initState()
  resetState: function(fn, scope, options)
  {
    this.initReady = false;
    this.state = [];
    delete this.extraVals;
    this.callback = fn.createDelegate(scope, options);
  },
  gsFilter: function(f) // send in '' to remove filter, nothing to get current filter, non-empty to set filter
  {
    if(f == null) return this.filter;
    else this.filter = f;
  },
  set: function(key, val) // only this one needs buffer, init doesn't, and clear shouldn't either
  {
    if(!this.dSet[key])
      this.dSet[key] = new Ext.util.DelayedTask(function(val) {
        Ext.Ajax.request({
          url: this.url,
          params: {appName: this.appName, action: 'set', key: key, value: this.encodeValue(val)},
          failure: function(){if(!this.failed) Ext.Msg.alert('Preference Saving Error', 'Failed to save (' + key + ':' + val + '). Further failure will not be reported again'); this.failed = true;},
          scope: this
        });
        this.parent.set.call(this, key, val);
        this.dSet[key] = undefined; // cleans up function refs (useful if there are many unique keys)
      }, this);
    this.dSet[key].delay(500, null, null, [val]);
  },
  clear: function(key)
  {
    Ext.Ajax.request({
      url: this.url,
      params: {appName: this.appName, action: 'clear', key: key},
      failure: function(){if(!this.failed) Ext.Msg.alert('Preference Saving Error', 'Failed to clear (' + key + '). Further failure will not be reported again'); this.failed = true;},
      scope: this
    });
    this.parent.clear.call(this, key);
  }
});