// create one commentmanager per tooltip
BMS.AG.ML.DynamicViewer.CommentManager = Ext.extend(Ext.TabPanel,
{
  rootURL: BMS.AG.ML.IMVConfig.editURL,
  constructor: function(conf)
  {
    Ext.apply(this, conf); // viewer is required, url is no longer needed
    if(!conf.width) conf.width = 436; // this strange width-got-lost issue happened on one user's computer.
    this.currentUser = this.viewer.currentUser;
    var tmp = new Array(this._setGrid()); // should we only add comment tab for DNA? (because col tips are recorded based on position only, so we can't allow two coordinate systems)
    if(this.info.track.type != 'base' && !BMS.AG.ML.IMVConfig.traceOff)
      tmp.push(this._setTrace());
    if(this.info.type == 'row')
    {
      tmp.push(this._setSeq());
      tmp.push(this._setMutSeq());
    }
    else
      tmp.push(this._setStatsGrid());
    if(this.info.track.group && this.info.track.type != 'base' && this.info.type != 'row')
      tmp.push(this._setStatsGrid(true));
    BMS.AG.ML.DynamicViewer.CommentManager.superclass.constructor.apply(this,
    [{
      activeTab: 0,
//       plain: true,
      defaults: {autoScroll: true},
      resizable: true,
      items: [
        {
          title: 'Info',
          html: this.info.h + this.getCommentTable() + (this.info.ihtml? '<br>' + this.info.ihtml : ''),
          height: this.conf.height
        }, tmp
      ]
    }]);
  },
  getCommentTable: function() {
    if(this.data)
    {
      var good = [], rows = ['<br><br><table class="x-tip-body commentTable"><tr><th>Context</th><th>Tag</th><th>Value</th></tr>'];
      for(var i = 0; i < this.data.length; i++)
      {
        var d = this.data[i];
        if(/^User /.test(d[2]) ||
          (d[2] == 'CSF' && /^(Contig|Amplicon|NidBarCode|Requisition)$/.test(d[0])))
          good.push(d);
      }
      good.sort(function(a,b){return a[2]>b[2] ? 1 : a[2]<b[2] ? -1 : (a[0]>b[0] ? 1 : a[0]<b[0] ? -1 : 0)});
      for(var i = 0; i < good.length; i++)
      {
        rows.push('<tr><td>'+good[i][2]+'</td><td>'+good[i][0]+'</td><td>'+good[i][1]+'</td></tr>');
      }
      return good.length? rows.join('\n') + '</table>' : '';
    }
    return '';
  },
  getSeqHtml: function(rate)
  {
    var aseq = this.info.track.getAlignSeq(rate), seq = this.info.track.getSeq(rate);
    var partseq = aseq.substr(this.info.tstart - 1, this.info.tend - this.info.tstart + 1);
    return '<b>Minimum Mutation % used to generate this seq: '+ (rate === undefined? 20:rate) +'</b>\n<br>Complete sequence for this track (without gap):<br><textarea class="fullseq" spellcheck="false">&gt;' + this.info.name + '\n' + seq + '</textarea><br>' +
            'Sequence for this track (with gap):<br><textarea class="fullseq" spellcheck="false">&gt;' + this.info.name + '\n' + aseq + '</textarea><br>' +
            'Sequence in this window (no gap):<br><textarea class="partseq" spellcheck="false">&gt;' + this.info.name + '(' + this.info.tstart + '-' + this.info.tend + ')\n' + partseq.replace(/- /g,'') + '</textarea><br>' +
            'Sequence in this window (with gap):<br><textarea class="partseq" spellcheck="false">&gt;' + this.info.name + '(' + this.info.tgstart + '-' + this.info.tgend + ')\n' + partseq + '</textarea>';
  },
  _setSeq: function()
  {
    this.seq = new Ext.Panel({
        title: 'FastaSeq',
        autoScroll: true,
        height: this.conf.height,
        width: this.conf.width, // needed for IE
        html: this.getSeqHtml(),
        listeners: {
          scope:this,
          activate: this.updateSeq
        },
        tbar: [
          {
            text: 'Refresh Sequences',
            tooltip: 'Press this button to refresh the sequences after you changed the "Min Mutation %" option in the "Mutation Filters" on the left side.',
            tooltipType: 'title',
            scope: this,
            handler: this.updateSeq
          }
        ]
      });
    return this.seq;
  },
  updateSeq: function() {
    var p = this.seq, rate = MutationViewer.getMinRate();
    p.body.update(this.getSeqHtml(rate));
  },
  _setMutSeq: function()
  {
    this.mutSeq = new Ext.Panel({
      title: 'MutantSeq',
      autoScroll: true,
      height: this.conf.height,
      width: this.conf.width, // needed for IE
      listeners: {
        scope:this,
        activate: this.updateMutSeq
      },
      tbar: [
        {
          text: 'Refresh Sequences',
          tooltip: 'Press this button to refresh the sequences after you changed the "Min Mutation %" option in the "Mutation Filters" on the left side.',
          tooltipType: 'title',
          scope: this,
          handler: this.updateMutSeq
        }
      ]
    });
    return this.mutSeq;
  },
  updateMutSeq: function() {
    var p = this.mutSeq, rate = MutationViewer.getMinRate();
    var seq = this.info.track.assembleMutSeq(rate);
    p.body.update('<textarea style="width:99%;height:99%" wrap="off" spellcheck="false">Minimum Mutation % used to generate this seq: '+ (rate === undefined? 20:rate) +'\n&gt;' + this.info.name + '\n' + seq + '</textarea>');
  },
  _setTrace: function()
  {
    this.trace = new Ext.Panel({
        title: 'Trace',
        autoScroll: true,
        height: this.conf.height,
        width: this.conf.width, // needed for IE
        listeners: {
          scope:this,
          activate:this._getTraceInfo,
          afterrender:function(p){
            p.body.on('contextmenu', function(e) {
              if(e.target.tagName == 'IMG')
              {
                e.preventDefault();
                e.stopPropagation();
                // find the base that was clicked
                var b = this.traceimage.coords.baseBounds, ib = e.target.getClientRects()[0],
                    x = e.xy[0], y = e.xy[1];
                if(!ib.height) // IE8
                {
                  var ia = {};
                  for(var i in ib) ia[i] = ib[i];
                  ib = ia;
                  ib.height = ib.bottom - ib.top + 1;
                  ib.width = ib.right - ib.left + 1;
                }
                for(var i = 0; i < b.length; i++)
                {
                  var a = b[i][0], c = b[i][1];
                  if(a[0] + ib.left <= x && x <= a[2] + ib.left &&
                     a[1] + ib.top <= y && y <= a[3] + ib.top)
                  {
                    // hilight the clicked base
                    var parent = e.target.parentNode;
                    parent.style.position = 'relative';
//                     this.body.dom.style.position = 'relative';
                    var div = document.createElement('div');
                    div.style.left = a[0] - 1;
                    div.style.width = a[2] - a[0] + 1;
//                     div.style.top = 0 - ib.height;
                    div.style.height = ib.height;
//                     div.style.position = 'relative';
                    div.style.top = 0;
//                     div.style.height = this.body.getHeight() - 3;
                    div.style.position = 'absolute';
                    div.style.border = '1px solid red';
                    parent.appendChild(div);
//                     this.body.dom.appendChild(div);
                    // now create the editor window
                    var win = new BMS.AG.ML.DynamicViewer.BaseEditor({
                      x: x + 5,
                      y: y + 5,
                      pos: c,
                      comParent: this,
                      traceimage: this.traceimage,
                      track: this.info.track.srcId? // is protein, need to get its corresponding dna track
                             this.viewer.trackManager.getTrack(this.info.track.srcId) : this.info.track,
                      rootURL: this.rootURL
                    });
                    win.show();
                    win.on('close',function(){parent.removeChild(div)},this);
                    break;
                  }
                }
              }
            }, this);
          }
        }
      });
    return this.trace;
  },
  _getTraceInfo: function(p)
  {
    if(!this.traceimage)
    {
      var mask = new Ext.LoadMask(p.id, {msg:'Retrieving trace image...'});
      mask.show();
      if(this.rootURL)
      {
        Ext.Ajax.request({
          scope: this,
          url: this.rootURL + '?agent=imb&id=' + this.info.track.id + '&mode=image&format=json'
               + ((this.info.type == 'cell')? '&cenbase=' + this.info.tstart : '') // cenbase means the trace returned should be centered around and highlight this base
               + ((dynamicViewer && dynamicViewer.timestamp)? '&timestamp=' + dynamicViewer.timestamp : ''),
          callback: function(o,s,r) {
            if(s)
            {
              var res = Ext.decode(r.responseText);
              if(res.png && res.png.length)
              {
                this.traceimage = res.png[0];
                p.update('<img src="'+ this.traceimage.url +'" />');
              }
              else
              BMS.AG.ML.Util.alert('Error', (res.error || 'Trace image info from server is corrupted!') + ' Technical details:\n\n' + (JSON && JSON.stringify? JSON.stringify(res || {}, null, 2) : Ext.encode(res || {})))
            }
            else
              BMS.AG.ML.Util.alert('Error','Failed to retrieve trace from server!');
            mask.hide();
          }
        });
      }
    }
  },
  _setGrid: function()
  {
    this.commentStore = new Ext.data.ArrayStore(this.conf.storeFields);
    if(this.data) this.commentStore.loadData(this.data.sort(function(a,b){return a[2]>b[2] ? 1 : a[2]<b[2] ? -1 : (a[0]>b[0] ? 1 : a[0]<b[0] ? -1 : 0)}));
    this.type = this.info.type == 'row'? 'Row' : this.info.type == 'col'? 'Column' : 'Cell';
    this.grid = new Ext.grid.EditorGridPanel({
        title: this.type + ' Comment',
        store: this.commentStore,
        cm: this.conf.getColModel(),
        stripeRows: true,
        height: this.conf.height,
        width: this.conf.width, // needed for IE
        frame: true,
        autoScroll: true,
        autoExpandColumn: this.conf.autoExpandColumn,
        selModel: new Ext.grid.RowSelectionModel(),
        tbar: [
          {
            text: 'Add Comment',
            scope: this,
            handler : this.createComment
          },
          {
            text: 'Delete Comment',
            scope: this,
            handler : this.removeComment
          }
        ],
        listeners: {beforeedit:this.checkPermission, afteredit:this.updateComment,scope:this}
    });
    this.grid.getTopToolbar().grid = this.grid;
    return this.grid;
  },
  _setStatsGrid: function(useGroup)
  {
    var tmp = new BMS.AG.ML.DynamicViewer.StatsGrid({
      height: this.conf.height,
      width: this.conf.width, // needed for IE
      useGroup: useGroup,
      callback: useGroup? null : { scope: this, fn: this.filterBases },
      listeners: {activate:this._updateStats, scope:this}
    });
    if(useGroup) this.groupStatsGrid = tmp;
    else this.statsGrid = tmp;
    return tmp;
  },
  _updateStats: function(p)
  {
    var useGroup = p.title == 'Group Stats';
    var stats = this.viewer.trackManager.positionStats(this.info.track.tclass,
                  this.info.tgstart, useGroup? this.info.track.group : null);
    var stat = stats[this.info.track.tclass];
    var data = new Array();
    for(var i in stat.all)
    {
      var s = stat.all[i];
      data.push(new Array((i == stat.base)? '*'+i : i, s.count, s.min, s.max, s.med, s.sd));
    }
    // special char
    data.sort(function(a,b){return a[0]==' '?1:b[0]==' '?-1:b[1]-a[1]||b[4]-a[4]}); //sort by count then median
    p.store.loadData(data);
  },
  filterBases: function()
  {
    var rows = this.statsGrid.getSelectionModel().getSelections();
    if(!rows) return;
    if(!rows.length) BMS.AG.ML.Util.alert('Warning','You must select some row(s) first!');
    else
    {
      var bases = {};
      for(var i = 0; i < rows.length; i++)
        bases[rows[i].data.base.replace('*','')] = 1;
      var info = this.viewer.getInfo(this.td.dom);
      this.viewer.filterBases(this.info.track.tclass, this.info.tgstart, bases);
      var id = this.viewer.id;
      setTimeout(function(){var table=Ext.get(id).dom;BMS.AG.ML.Util.addClass(table.rows[info.baseRowIndex].children[info.baseCellIndex], 'filtered')}, 20) ;//after filter data reload these indices should be the same, we need timeout longer than drawTracks otherwise it's updated before data reload and will be removed
    }
  },
  createComment: function()
  {
    var p = new this.conf.Comment(this.conf.getInitComValue(this.currentUser));
    this.grid.stopEditing();
    this.commentStore.insert(0, p);
    this.grid.startEditing(0, 2);
  },
  // only allows removal of one's own comments
  removeComment: function()
  {
    var rows = this.grid.getSelectionModel().getSelections();
    if(!rows) return;
    if(!rows.length) BMS.AG.ML.Util.alert('Warning','You must select some of your own comment(s) first!');
    else
    {
      this.rows = new Array();
      for(var i = 0; i < rows.length; i++)
        if(rows[i].data.context == this.currentUser)
          this.rows.push(rows[i]);

      if(!this.rows.length) BMS.AG.ML.Util.alert('Warning','You are only allowed to remove your own comments!');
      else
        BMS.AG.ML.Util.confirm('Delete Selection','Remove selected comment(s) of yours from the database?',
          this._removeComment,this);
    }
  },
  // delegate update as CommentManager knows nothing about comment formats
  updateComment: function(e)
  {
    var changes = e.record.getChanges();
    this.viewer.updateComment(e, this.info);
    replaceColor(this.td.dom, 'lowlite');
  },
  // only allows editing of one's own comments
  checkPermission : function (e)
  {
    if(e.record.data.context != this.currentUser)
    {
      BMS.AG.ML.Util.alert('Info', 'You can only edit your own comment!');
      return false;
    }
    return true;
  },
  _removeComment: function(button)
  {
    if(button == 'no') { return }
    for(var i = 0; i < this.rows.length; i++)
    {
      // fire an Ajax call to remove the record
      this.viewer.removeComment(this.rows[i], this.info);
      this.commentStore.remove(this.rows[i]);
    }
    if(this.commentStore.data.length == 0)
      restoreColor(this.td.dom, 'lowlite');
  }
});
