BMS.AG.ML.DynamicViewer.StatsWizard = Ext.extend(Ext.Panel, {
  constructor: function(config) {
    Ext.apply(this, config, {
      layout: 'card',
      activeItem: 0,
      bodyStyle: 'padding:15px',
      defaults: { height: config.totalHeight - 56 },
      id: 'card',
      bbar: [
        '->',
        {
          text: '&laquo; Previous',
          handler: this.movePage.createDelegate(this, [-1]),
          disabled: true
        },
        {
          text: 'Next &raquo;',
          handler: this.movePage.createDelegate(this, [1])
        }
      ],
      items: [{
        id: 'card-0',
        xtype: 'form',
        border:false,
        defaults: { border: false },
        items: [
          {
            html: '<span style="color:red;font-weight:bold;margin-bottom:15px;">Warning: </span><span style="margin-bottom:15px;">This tool provides accurate data ONLY if your sequence data come from clonal sequencing! Stats/sequences for sequencing done on samples with pooled (viral) sequence populations most likely do not accurately reflect stats/sequence populations, although one might find the stats helpful as a rough guide.</span>'
          },
          { height: 5 },
          { height: 1, border: true },
          { height: 10 },
          {
            layout: 'table',
            viewConfig: { columns: 2 },
            items: [
              {
                width: 210,
                border: false,
                html: '<span style="font:12px tahoma,arial,helvetica,sans-serif;">Please enter the positions below and</span>'
              },
              {
                xtype: 'combo',
                emptyText: 'please select a track',
                store: [['dna','DNA'], ['prot','Protein']]
              }
            ]
          },
          { height: 10 },
          {
            anchor: '85% 65%',
            hideLabel: true,
            style: 'margin-left:20px;',
            id: 'positions',
            emptyText: 'Format: 1,3,25-30,79...',
            xtype: 'textarea'
          }
        ]
      },
      {
        id: 'card-1',
        xtype: 'tabpanel',
        activeItem: 0,
        items: [
          {
            xtype: 'statsgrid',
            height: config.totalHeight - 56,
            width: config.totalWidth - 34
          },
          {
            title: 'Stats (Copy to Excel)',
            layout: 'fit',
            items: {
              xtype: 'textarea',
              border: false
            }
          }
        ]
      }]
    });
    BMS.AG.ML.DynamicViewer.StatsWizard.superclass.constructor.call(this);
  },
  movePage: function(step)
  {
    var pos = this.get(0).get(6).getValue(), track = this.get(0).get(4).get(1),
        ttype = track.getValue();
    // double check form input
    if(step > 0)
    {
      if(pos.match(/^[,\s-]*$|^Format:/))
      {
        BMS.AG.ML.Util.alert('Error', 'Position input cannot be empty!');
        return;
      }
      if(pos.match(/[^0-9,\s-]/))
      {
        BMS.AG.ML.Util.alert('Error', 'Position input was in the wrong format! Acceptable format is: 1,3,25-30,79...');
        return;
      }
      if(!ttype)
      {
        BMS.AG.ML.Util.alert('Error', 'You must select a track!');
        track.focus();
        return;
      }
    }
    var layout = this.getLayout(), bar = this.toolbars[0],
        current = this.getIndex(layout.activeItem), future = current + step,
        next = bar.items.items[2], prev = bar.items.items[1];
    if(future == this.items.items.length - 1) next.disable();
    else next.enable();
    if(future == 0) prev.disable();
    else prev.enable();

    // pre-process the positions and add gaps
    var allPos = [], gapped = [], base = this.viewer.trackManager.getBaseForClass(ttype);
    var posarr = pos.split(/[,\s]+/);
    for(var i = 0; i < posarr.length; i++)
    {
      var t = posarr[i].split(/-/);
      if(t.length > 1)
      {
        for(var j = t[0]; j <= t[1]; j++)
          allPos.push(base.addGap(j-0));
      }
      else allPos.push(base.addGap(t[0]-0));
    }
    // now get seqs and stats
    var stats = this.viewer.trackManager.positionStats(ttype, allPos);
    var stat = stats[ttype], data = [], seqstr = [];
    for(var i in stat.all)
    {
      var s = stat.all[i];
      data.push([(i == stat.base)? '*'+i : i, s.count, s.min, s.max, s.med, s.sd]);
    }
    data.sort(function(a,b){return a[0]==' '?1:b[0]==' '?-1:b[1]-a[1]||b[4]-a[4]}); //sort by count then median
    for(var i = 0; i < data.length; i++)
      seqstr.push(data[i].join('\t'));

    // now update data
    var p = this.get(1).get(0), ta = this.get(1).get(1).get(0);
    p.store.loadData(data);
    ta.setValue('Track:' + ttype + ', Positions:' + pos + '\n' +
                ['nt/aa', 'Count', 'Min %', 'Max %', 'Med %', 'SD'].join('\t') + '\n' +
                seqstr.join('\n'));

    // now show the stats
    layout.setActiveItem(future);
  },
  getIndex: function(item)
  {
    for(var i = 0; i < this.items.items.length; i++)
      if(this.items.items[i] === item)
        return i;
  }
});
Ext.reg("statswizard", BMS.AG.ML.DynamicViewer.StatsWizard);