///////////////////////////////////////////////////////////////////////////////////////////
// Purpose: Provide a control for special html-table-based viewers (the performance is
//          best with sparse features on map or smaller controller configuration! Otherwise
//          you should try to use FF instead of IE or Chrome which is fast at small tables but
//          bad at large ones)
// How to use: 1. Include Ext.
//             2. Include this class and its css.
//             3. Create a new instance 'var mv = BMS.AG.ML.viewerController();
// Note: this viewer assumes that entire data area is bigger than current view window! (otherwise display looks a bit weird)
// Author: Mingyi Liu

BMS.AG.ML.ViewerController = function(config)
{
  if(!config)
    alert('Must pass in config that specifies data, callback and scope!');
  this.data = config.data;
  this.callback = config.callback;
  this.scope = config.scope;
  this.cellSize = BMS.AG.ML.Util.getCSSRule('div.viewerController .vcmap td', 'width', {isNum:true});
  var defaults = { id:'AGML_vc', renderTo: 'viewcontrol', hidden: true };
  if(!this.config) this.config = {};
  Ext.apply(this.config, config, defaults);
};
BMS.AG.ML.ViewerController.prototype =
{
  updateView: function()
  {
    this.getSize();
    if(!this.viewDiv) this.viewDiv = new Ext.Panel(this.config);
    else
    {
      this.viewDiv.setWidth(this.config.width);
      this.viewDiv.setHeight(this.config.height);
    }
    if(!this.bodyEl)
    {
      this.bodyEl = Ext.get(this.viewDiv.body.id);
      this.bodyEl.on('click', this._repositionRect, this);
    }
    this.show();
    if(this.data)
    {
      this._renderData();
      this._positionRect();
      this._initDD();
      this.initiated = true;
    }
  },
  // get the container size and calculate width/height from it
  getSize: function()
  {
    if(!this.container) this.container = Ext.get(this.config.renderTo);
    this.config.width = this.container.getWidth();
    this.config.height = this.container.getHeight();
  },
  moveCross: function(x1, y1)
  {
    var x = Math.round(x1 * this.xscale), y = Math.round(y1 * this.yscale);
    if(this.vl) this.vl.setLeft(x - 1);
    if(this.hl) this.hl.setTop(y - 1);
  },
  show: function()
  {
    this.viewDiv.show();
  },
  hide: function()
  {
    this.viewDiv.hide();
  },
  updateCallerView: function(e)
  {
    var v = Ext.get('vcsmall').dom;
    this.rect[0] = (this.cellSize > 1)? Math.round(v.offsetLeft / this.cellSize) : v.offsetLeft;
    this.rect[1] = (this.cellSize > 1)? Math.round(v.offsetTop / this.cellSize) : v.offsetTop;
    this._updateCaller();
  },
  // rect: [x, y, w, h]
  updateRect: function(rect)
  {
    this._setRect(rect);
    this._positionRect();
  },
  setData: function(data)
  {
    this.data = data;
  },
  // this fn is used to white/recolor some features that exist (meaning featData must match this.data.feat except for coloring info)
  repaintFeatures: function(featData)
  {
    // process visibility data
    this.repaintFeat = {};
    if(this.data.featOnly)
    {
      for(var i = 0; i < featData.length; i++)
        this._setRepaintFeat(featData[i][0], featData[i][1]);
    }
    else
    {
      for(var i = 0; i < featData.length; i++) // y
        for(var j = 0; j < featData[i].length; j++) //x
          if(featData[i][j])
            this._setRepaintFeat(i, j);
    }

    var map = Ext.get('vcmap');
    for(var j = 1; j <= this.y; j++)
    {
      for(var i = 1; i <= this.x; i++)
        if(this.feat[i] && this.feat[i][j])
        {
          var row = this.feat2cell[i][j][1], col = this.feat2cell[i][j][0];
//           var cell = map.select('tr:nth('+row+')').select('td:nth('+col+')');
//           cell.setVisibilityMode(Ext.Element.VISIBILITY);
//           if(this.repaintFeat[i] && this.repaintFeat[i][j]) cell.setVisible(true);
//           else cell.setVisible(false);
          var cell = map.dom.rows[row].children[col];
          if(this.repaintFeat[i] && this.repaintFeat[i][j]) cell.style.visibility = 'visible';
          else cell.style.visibility = 'hidden';
        }
    }
  },
  // private functions
  // data should be of format { len:[6,4], rect:[1,1,3,2], featOnly:false, feat:[[1,0,0],[1,0,1]] } // transmit entire array
  //                       or { len:[6,4], rect:[1,1,3,2], featOnly:true, feat:[[1,1],[1,2],[3,2]] } // only record the [x,y] that HAS feat, index starts at 1
  // if the color of features should change according to value (0-1), then
  // data should be of format { len:[6,4], rect:[1,1,3,2], color:{fn:myColor,scope:this}, featOnly:false, feat:[[0.25,0,0],[0.4,0,0.75]] } // transmit entire array
  //                       or { len:[6,4], rect:[1,1,3,2], color:{fn:myColor,scope:this}, feat:[[1,1,0.25],[1,2,0.4],[3,2,0.75]] } // only record the [x,y] that HAS feat, index starts at 1
  // fn is optional (scope is optional and if it's needed at all depends on definition of fn).
  // fn should take the 0-1 value and generate appropriate feature color
  _scale: function()
  {
    this.x = this.bodyEl.getWidth() - 2; // offset border
    this.y = this.bodyEl.getHeight() - 2; // offset border
    this.xscale = this.x / this.data.len[0];
    this.yscale = this.y / this.data.len[1];
    this._setRect(this.data.rect);
    // now convert data into suitable format & calculate value if needed
    var t = {};
    this.feat = {};
    if(this.data.featOnly)
    {
      for(var i = 0; i < this.data.feat.length; i++)
        this._setFeat(this.data.feat[i][0], this.data.feat[i][1], (this.data.color)? this.data.feat[i][2] : 1);
    }
    else
    {
      for(var i = 0; i < this.data.feat.length; i++) // y
        for(var j = 0; j < this.data.feat[i].length; j++) //x
          if(this.data.feat[i][j])
            this._setFeat(i, j, this.data.feat[i][j]);
    }
  },
  // double click repositions rectangle
  _repositionRect: function(e, t)
  {
    if(this.justDragged) // endDrag just dealt with the event
    {
      this.justDragged = false;
      return;
    }
    var xy = e.getXY(), xy1 = this.bodyEl.getXY();
    var x = xy[0] - xy1[0] - Math.round(this.rect[2] / 2);
    var y = xy[1] - xy1[1] - Math.round(this.rect[3] / 2);
    var maxx = this.bodyEl.getWidth() - this.rect[2] - 1;
    var maxy = this.bodyEl.getHeight() - this.rect[3] - 1;
    this.rect[0] = (x < 0)? 0 : (x > maxx)? maxx : x;
    this.rect[1] = (y < 0)? 0 : (y > maxy)? maxy : y;
    this.rectEl.setLeftTop(this.rect[0], this.rect[1]);
    this._updateCaller();
  },
  _updateCaller: function()
  {
    this.callback.apply(this.scope, [Math.round((this.rect[0] + 1)/ this.xscale), Math.round((this.rect[1] + 1) / this.yscale)]);
  },
  _setRect: function(rect)
  {
    if(!this.rect) this.rect = new Array();
    this.rect[0] = Math.round(rect[0] * this.xscale);
    this.rect[2] = Math.round(rect[2] * this.xscale);
    this.rect[1] = Math.round(rect[1] * this.yscale);
    this.rect[3] = Math.round(rect[3] * this.yscale);
  },
  _positionRect: function()
  {
    this.rectEl.setLeftTop(this.rect[0] - 1, this.rect[1] - 1);
    this.rectEl.setWidth(this.rect[2]);
    this.rectEl.setHeight(this.rect[3]);
  },
  _setFeat: function(i1, j1, val)
  {
    var i = Math.round(i1 * this.xscale), j = Math.round(j1 * this.yscale);
    if(! this.feat[i]) this.feat[i] = {};
    this.feat[i][j] = 0;
    if(this.data.color)
    {
      this.feat[i][j] += val;
      if(val > 1) val = 1;
    }
    else this.feat[i][j] = 1;
  },
  _setRepaintFeat: function(i1, j1, val)
  {
    var i = Math.round(i1 * this.xscale), j = Math.round(j1 * this.yscale);
    if(! this.repaintFeat[i]) this.repaintFeat[i] = {};
    this.repaintFeat[i][j] = 1;
  },
  _getFeatColor: function(val)
  {
    var color = this.featColor;
    if(!color) color = BMS.AG.ML.Util.getCSSRule('div.viewerController .vcmap td.feat', 'backgroundColor', {isColor:true});
    if(!color) color = [255, 0, 0];
    if(!this.featColor) this.featColor = color;
    var max = (color[0] > color[1])? ((color[0] > color[2])? 0 : 2) : ((color[1] > color[2])? 1 : 2);
    var d = new Array();
    for(var i = 0; i < 3; i++)
    {
      if(i == max) d.push(color[i]);
      else d.push(Math.floor((255 - color[i]) * (1 - val)));
    }
    return BMS.AG.ML.Util.getColor(d);
  },
  // create the data table as needed
  _renderData: function()
  {
    // this combined with insertHtml() for table makes DD not working
//     this.rectEl = this.bodyEl.update('<div id="vcsmall" class="vcsmall"></div>');
    this._scale();
    var rows = new Array();
    rows.push('<div id="vcsmall" class="vcsmall"></div><table id="vcmap" class="vcmap"><div id="vcvertline" class="vcvertline"></div><div id="vchoriline" class="vchoriline">&nbsp;</div><tr class="nodisplay">');
    for(var i = 1; i <= this.x; i++) // for IE this hidden row has to be in front of all others
      rows.push('<td></td>');
    rows.push('</tr>\n');
    if(this.data.color)
    {
      if(this.data.color.fn) this.color = this.data.color;
      else this.color = {fn:this._getFeatColor, scope:this};
    }
    // for IE it's crucial to use the Array.join instead of string concat (6 fold increase in speed for IE while FF same)
    this.feat2cell = {};
    for(var j = 1; j <= this.y; j++)
    {
      rows.push('<tr>');
      var lastx = 1, found = false, td = 0;
      for(var i = 1; i <= this.x; i++)
        if(this.feat[i] && this.feat[i][j])
        {
          found = true;
          if(i > lastx)
          {
            var span = (i - lastx > 1)? ' colspan=' + (i - lastx) : '';
            rows.push('<td',span,'></td>');
            td++;
          }
          var color = (this.color)?  ' style="background-color:' +
                      this.color.fn.apply(this.color.scope, [this.feat[i][j]]) + ';"' :
                      ' class="feat"';
          rows.push('<td',color,'></td>');
          if(!this.feat2cell[i]) this.feat2cell[i] = {};
          this.feat2cell[i][j] = [td,j];
          td++;
          lastx = i + 1;
        }
      if(lastx <= this.x) rows.push('<td colspan=',(this.x - lastx + 1),'></td>'); // FF and IE has limit colspan <= 1000, shouldn't apply here
      rows.push('</tr>\n');
    }
    this.bodyEl.update(rows.join('') + '</table>');
    this.rectEl = Ext.get('vcsmall');
    this.tableEl = Ext.get('vcmap');
    this.vl = Ext.get("vcvertline");
    this.vl.setHeight(this.tableEl.getHeight());
    this.hl = Ext.get("vchoriline");
    this.hl.setWidth(this.tableEl.getWidth());
  },
  _initDD : function()
  {
    this.dd = new BMS.AG.ML.ViewerController.DD(this, this.rectEl);
  }
};
BMS.AG.ML.ViewerController.DD = function(vc, e, config)
{
  Ext.apply(this, config);
  this.div = e;
  this.vc = vc;
  BMS.AG.ML.ViewerController.DD.superclass.constructor.call(this, e.id, 'VCDD-'+e.id);
  this.setHandleElId(e.id);
  this.scroll = false;
};
Ext.extend(BMS.AG.ML.ViewerController.DD, Ext.dd.DD,
  {
    moveOnly:true,
    scroll:false,
    startDrag: function()
    {
      var t;
      if(!this.vc.config.title) t = (Ext.isIE)? {bottom:-1,right:-1,left:-1} : {bottom:-3,right:-1};
      else t = (Ext.isIE)? {bottom:0,right:-1,left:-1} : {bottom:-2,right:-1};
      this.constrainTo(this.vc.tableEl.id, t);
    },
    endDrag: function(e)
    {
      this.vc.updateCallerView(e);
      this.vc.justDragged = true;
    }
  });